local Settings = {};
Settings.EmailName = GetSetting("EmailName");
Settings.Days = GetSetting("Days");
Settings.EmailSubject = GetSetting("EmailSubject");
Settings.Site = GetSetting("Site");
local isCurrentlyProcessing = false;

function Init()
    LogDebug("Initializing Electronic Delivery Reminder addon.");
    RegisterSystemEventHandler("SystemTimerElapsed", "TimerElapsed");
end

function TimerElapsed(eventArgs)
	if not isCurrentlyProcessing then
		isCurrentlyProcessing = true;
		local connection;
		connection = CreateManagedDatabaseConnection();

		-- Attempt to connect to the database
		local succeeded, err = pcall(ConnectToDatabase, connection)
		if not succeeded then
			LogDebug("Error connecting to database.");
			OnError(err)
			isCurrentlyProcessing = false;
			connection:Dispose();
			return
		end
		
		-- Send Emails
		local succeeded, err = pcall(SendEmails, connection);
		if not succeeded then
			LogDebug("Error sending emails.");
			OnError(err);
		end
		
		connection:Dispose();
		isCurrentlyProcessing = false;
	else
		LogDebug("Still sending emails, skipping.");
	end
end

function ConnectToDatabase(connection)
	connection:Connect();
end

function SharedServerSupport(connection)
    connection.QueryString = "SELECT Value FROM Customization WHERE CustKey = 'SharedServerSupport' AND NVTGC = 'ILL'";
    local value = connection:ExecuteScalar();

    if (value == "Yes") then
        LogDebug('Shared Server Support enabled');
        sharedServerSupport = true;
    else
        LogDebug('Shared Server Support not enabled');
        sharedServerSupport = false;
    end
end

function SendEmails(connection)
    local usersTable = "Users";
    local nvtgc = "ILL";

    -- Determine if shared server support customer  
    SharedServerSupport(connection); 

    if sharedServerSupport then
        usersTable = "UsersAll";
        nvtgc = Settings.Site;
    end

    -- Get value of DaysBeforeElectronicDeliveryCleanup
    connection.QueryString = [[SELECT Value FROM Customization WHERE CustKey = 'DaysBeforeElectronicDeliveryCleanup' AND nvtgc = ']] .. nvtgc .. [[']];
    local cleanupDays = connection:ExecuteScalar();

    -- Determine which requests need an email sent
	connection.QueryString = [[
		SELECT t.TransactionNumber, u.Username, u.NVTGC FROM Transactions t
		INNER JOIN ]] .. usersTable .. [[ u ON t.Username = u.Username
		WHERE u.NVTGC = ']] .. nvtgc .. [['
		AND t.TransactionStatus = 'Delivered to Web'
		AND t.TransactionNumber IN (SELECT DISTINCT TransactionNumber FROM EmailCopies)
		AND t.TransactionNumber IN (SELECT DISTINCT TransactionNumber FROM History)
		AND t.TransactionNumber NOT IN (SELECT DISTINCT TransactionNumber FROM History WHERE TransactionNumber = t.TransactionNumber AND Entry = 'PDF Viewed')
		AND t.TransactionNumber IN (SELECT TOP 1 TransactionNumber FROM History WHERE TransactionNumber = t.TransactionNumber AND Entry = 'Email Added: Requested Item Delivered Electronically' AND (DATEADD(dd, ]] .. (cleanupDays - Settings.Days) .. [[, DateTime) < GETDATE()) ORDER BY DateTime DESC)
		AND t.TransactionNumber NOT IN (SELECT TransactionNumber FROM EmailCopies WHERE TransactionNumber = t.TransactionNumber AND Subject = ']] .. Settings.EmailSubject .. [[' AND DATEADD(dd, -1, EmailDate) < GETDATE())]]
    local transactions = connection:Execute();

    -- Loop thru tn number and send email for each one.
    for i =0, transactions.Rows.Count - 1 do
        LogDebug("Email Subject is: " ..Settings.EmailSubject);
        ExecuteCommand("SendTransactionEmail", {transactions.Rows:get_Item(i):get_Item(0), Settings.EmailName, Settings.EmailSubject});
        LogDebug("Reminder Email sent for: " .. transactions.Rows:get_Item(i):get_Item(0));
    end 
end

function OnError(e)
    if e == nil then
        LogDebug("OnError supplied a nil error");
        return;
    end

    if not e.GetType then
        -- Not a .NET type
        -- Attempt to log value
        pcall(function ()
            LogDebug(e);
        end);
        return;
    else
        if not e.Message then
            LogDebug(e:ToString());
            return;
        end
    end

    local message = TraverseError(e);

    if message == nil then
        message = "Unspecified Error";
    end

	LogDebug("An error occurred in the " .. scriptErrorEventArgs.ScriptName .. " addon in the " .. scriptErrorEventArgs.ScriptMethod .. " function: " .. message);
end

-- Recursively logs exception messages and returns the innermost message to caller
function TraverseError(e)
    if not e.GetType then
        -- Not a .NET type
        return nil;
    else
        if not e.Message then
            -- Not a .NET exception
            LogDebug(e:ToString());
            return nil;
        end
    end

    LogDebug(e.Message);

    if e.InnerException then
        return TraverseError(e.InnerException);
    else
        return e.Message;
    end
end
